import os
import click
import click_spinner
import podcast_summarizer.youtube as youtube
import podcast_summarizer.summarizer as summarizer
from shutil import which


@click.command()
@click.version_option()
@click.option(
    "--summary-file", "-s", default=None, help="The file to save the summary to."
)
@click.option(
    "--transcription-file",
    "-t",
    default=None,
    help="The file to save the transcription to.",
)
@click.option(
    "--audio-format",
    "-f",
    default=None,
    help="The format of the audio file. Defaults to mp3 when it cannot be infered.",
)
@click.option(
    "--language",
    "-l",
    default=None,
    help="The language code for the language of the podcast.",
)
@click.argument("podcast_url_or_path")
def cli(podcast_url_or_path, summary_file, transcription_file, audio_format, language):
    """Summarizes the podcast provided by the PODCAST_URL_OR_PATH.

    - PODCAST_URL_OR_PATH can be a URL or a path to a local audio file.

    - PODCAST_URL_OR_PATH can be a YouTube URL (https://www.youtube.com/watch?v=...)

    - PODCAST_URL_OR_PATH can also be a path to a local ".txt" file containing the transcription.
    """

    if not which("ffmpeg"):
        raise Exception("ffmpeg is not installed. Please install ffmpeg.")

    if not os.getenv("OPENAI_API_KEY"):
        raise Exception(
            "OPENAI_API_KEY is not set. Please add your API key to your environment variables."
        )

    if podcast_url_or_path.startswith("https://www.youtube.com/watch?v="):
        if not language:
            language = youtube.detect_language(podcast_url_or_path)

        podcast_url_or_path = youtube.extract_from_youtube(
            podcast_url_or_path, transcription_file, language
        )

    if podcast_url_or_path.lower().endswith(".txt") and os.path.isfile(
        podcast_url_or_path
    ):
        transcription = open(podcast_url_or_path, "r").read()
    else:
        with summarizer.openAudio(podcast_url_or_path, audio_format) as file:
            click.echo("🎙️ Transcribing...", nl=False)
            with click_spinner.spinner():
                transcription = summarizer.transcribe(file.name)
            click.echo("\r", nl=False)
            click.secho("🎙️ Transcription complete!", fg="green")

        if transcription_file:
            with open(transcription_file, "w") as transcription_file:
                transcription_file.write(transcription)
                click.secho("💾 Transcription saved to transcription.txt")

    click.echo("📝 Summarizing...", nl=False)
    with click_spinner.spinner():
        summary = summarizer.summarize(transcription, language)
    click.echo("\r", nl=False)
    click.secho("📝 Summary complete!", fg="green")

    if summary_file:
        with open(summary_file, "w") as summary_file:
            summary_file.write(summary)
            click.secho("💾 Summary saved to summary.txt", fg="green")

    click.echo("\n", nl=False)
    click.echo(summary)

    click.secho(
        "\nDo you have any questions about the podcast? (Or type 'exit' to quit)",
        fg="blue",
    )

    prompt = click.prompt(click.style(
        "\n> ", fg="bright_white"), prompt_suffix="")
    while True:
        if prompt.lower() == "exit":
            break
        click.echo(summarizer.ask_LLM(prompt))
        prompt = click.prompt(click.style(
            "\n> ", fg="bright_white"), prompt_suffix="")


if __name__ == "__main__":
    cli()



import os
import click
import webvtt
import yt_dlp


def extract_from_youtube(
    podcast_url_or_path, transcription_file=None, language=None
):
    download_options = get_download_options()

    if not language:
        language = detect_language(podcast_url_or_path)

    download_options["subtitleslangs"] = [language]

    with yt_dlp.YoutubeDL(download_options) as ydl:
        click.echo("📺 Looking for YouTube subtitles...")
        ydl.download(podcast_url_or_path)

        subtitles_file = f"yt-audio.{language}.vtt"
        if os.path.isfile(subtitles_file):
            vtt = webvtt.read(subtitles_file)

            if not transcription_file:
                transcription_file = "yt-audio.txt"
            if not transcription_file.endswith(".txt"):
                transcription_file += ".txt"

            create_transcription_from_subtitles(transcription_file, vtt)

            podcast_url_or_path = transcription_file

        else:
            download_options["skip_download"] = False
            with yt_dlp.YoutubeDL(download_options) as ydl:
                click.echo(
                    "📺 Extracting audio from YouTube video \
                        (may take a while)..."
                )
                ydl.download(podcast_url_or_path)
                podcast_url_or_path = "yt-audio.m4a"
                click.echo("\r", nl=False)
                click.secho(
                    "💾 Saved YouTube audio to yt-audio.m4a", fg="green")

    return podcast_url_or_path


def create_transcription_from_subtitles(transcription_file, vtt):
    with open(transcription_file, "w") as f:
        transcript = ""
        lines = []
        for line in vtt:
            lines.extend(line.text.strip().splitlines())

        previous = None
        for line in lines:
            if line == previous:
                continue
            transcript += "\n" + line
            previous = line

        f.write(transcript)

    click.echo("\r", nl=False)
    click.secho(
        f"💾 Saved YouTube transcription to {transcription_file}", fg="green"
    )


def detect_language(podcast_url_or_path):

    download_options = get_download_options()

    with yt_dlp.YoutubeDL(download_options) as ydl:
        click.echo("🌎 Detecting original language...")
        ydl.download(podcast_url_or_path)

        info = ydl.extract_info(podcast_url_or_path, download=False)

        language = next(
            format["language"]
            for format in info["formats"]
            if format["format_note"] == "Default"
        )

        if language:
            click.secho(
                f"🗣️ Detected original language: {language}", fg="green")
        else:
            language = "en"
    return language


def get_download_options():
    return {
        "format": "bestaudio/best",
        "skip_download": True,
        "outtmpl": "yt-audio",
        "quiet": True,
        "writesubtitles": True,
        "writeautomaticsub": True,
        "postprocessors": [
            {
                "key": "FFmpegExtractAudio",
                "preferredcodec": "m4a",
            }
        ],
    }



import os
import openai
import urllib.request
import tempfile
import tiktoken
import whisper
import logging

# Configure logging
logging.basicConfig(level=logging.INFO, format="%(levelname)s - %(message)s")

messages = []
SUPPORTED_AUDIO_FORMATS = ["m4a", "mp3", "webm", "mp4", "mpga", "wav", "mpeg"]
TOKEN_LIMIT = 8192


def openAudio(url_or_path, audio_format):
    '''Opens an audio file from a URL or path.'''

    if os.path.isfile(url_or_path):
        return open(url_or_path, "rb")
    try:
        if not audio_format:
            audio_format = os.path.splitext(url_or_path)[1]
            if not audio_format or audio_format not in SUPPORTED_AUDIO_FORMATS:
                audio_format = "mp3"

        audio_file = tempfile.NamedTemporaryFile(suffix=audio_format)
        with urllib.request.urlopen(url_or_path) as audio:
            audio_file.write(audio.read())
        return audio_file
    except ValueError:
        raise Exception("Invalid URL or path.")


def transcribe(mp3_path):
    model = whisper.load_model("tiny")
    result = model.transcribe(mp3_path, fp16=False)

    return result["text"]


def summarize(text, language=None):
    '''Summarizes the podcast using the LLM model.'''

    if num_tokens_from_string(text) > TOKEN_LIMIT:
        # Split text into multiple parts and summarize each part
        text = compress_text(text, language)

    if language:
        return ask_LLM(
            f"Please summarize the following podcast transcription,\
                respond in \"{language}\": {text}"
        )
    else:
        return ask_LLM(
            f'Please summarize the following podcast transcription,\
                respond in the same language as the transcription: {text}'
        )


def compress_text(text, language=None):
    '''Compresses the text by recursively summarizing it in two parts.'''

    encoding = tiktoken.encoding_for_model("gpt-3.5-turbo")
    tokens = encoding.encode(text)

    # Split token list in two
    first_tokens = tokens[: len(tokens) // 2]
    second_tokens = tokens[len(tokens) // 2:]

    if len(first_tokens) > TOKEN_LIMIT:
        first_tokens = encoding.encode(
            compress_text(encoding.decode(first_tokens)))

    if len(second_tokens) > TOKEN_LIMIT:
        second_tokens = encoding.encode(
            compress_text(encoding.decode(second_tokens)))

    # Summarize each half
    first_half = compress_through_LLM(encoding.decode(first_tokens), language)
    second_half = compress_through_LLM(
        encoding.decode(second_tokens), language)

    # Combine summaries
    return first_half + '' + second_half


def compress_through_LLM(text, language=None):
    '''Summarizes the text using the LLM model.'''
    if language:
        return ask_LLM(
            f"Please summarize the following podcast transcription,\
                respond in \"{language}\": {text}", False)
    else:
        return ask_LLM(
            f"Please make a summarized version of this podcast transcription:\
              {text}", False)


def ask_LLM(message, chat=True):
    if chat:
        global messages
    else:
        messages = []

    openai.api_key = os.getenv("OPENAI_API_KEY")
    if not openai.api_key:
        logging.error("OPENAI_API_KEY not set", exc_info=True)

    if num_tokens_from_string(message) > TOKEN_LIMIT:
        logging.error("Message exceeds token limit", exc_info=True)

    messages += [
        {
            "role": "user",
            "content": message,
        }
    ]

    try:
        completion = openai.ChatCompletion.create(
            model="gpt-3.5-turbo-16k",
            messages=messages,
            temperature=0.7,
        )
    except openai.error.RateLimitError as e:
        logging.error("RateLimitError occurred:", exc_info=True)
        logging.error("RateLimitError: %s", e)
        logging.error("Error details: %s", e.__dict__)

    response = completion.choices[0].message.content
    return response


def num_tokens_from_string(string, encoding_name="gpt-3.5-turbo"):
    encoding = tiktoken.encoding_for_model(encoding_name)
    num_tokens = len(encoding.encode(string))
    return num_tokens
